import click


class AddressParamType(click.ParamType):
    name = 'address'

    def __call__(self, value, param=None, ctx=None):
        if value is None:
            return (None, None)
        return self.convert(value, param, ctx)

    def convert(self, value, param, ctx):
        if ':' in value:
            host, port = value.split(':', 1)
            port = int(port)
        else:
            host = value
            port = None
        return host, port


Address = AddressParamType()


@click.group()
def run():
    "Run a service."

@run.command()
@click.option(
    '--debug', '-d', default=False,
    help='The debug option for web'
)
def web(debug):
    from seed.services.app import application
    from seed.services.wsgi import SeedWSGIHttpServer

    if debug:
        application.run()
    else:
        SeedWSGIHttpServer(application).run()



import json
from seed.libs.data_access.formatters.line import LineFormatter


class FunnelFormatter(LineFormatter):
    def __init__(self, *args, **kwargs):
        super(LineFormatter, self).__init__(*args, **kwargs)

    def format_data(self):
        result = {"series": []}
        ret = {"data": []}
        for dimenstr, datas in self.data.items():
            info = {}
            dimendict = json.loads(dimenstr)
            namelist = [dimen.get("dimension", "") for dimen in self.dimensions]
            infoname = []
            for name in namelist:
                infoname.append(str(dimendict.get(name, "")))

            info["name"] = "-".join(infoname)
            info["value"] = 0
            for key, value in datas.items():
                if isinstance(value, (float, int)):
                    info["value"] += value
            ret["data"].append(info)

        result["series"].append(ret)

        return result



from __future__ import absolute_import

import os
import sys
import shutil
import traceback

from distutils import log

from .base import BaseBuildCommand


class BuildAssetsCommand(BaseBuildCommand):
    user_options = BaseBuildCommand.user_options + [

    ]

    description = 'build static media assets'

    def initialize_options(self):
        BaseBuildCommand.initialize_options(self)

    def _build(self):
        try:
            self._build_static()
        except Exception:
            traceback.print_exc()
            log.fatal(
                'unable to build Seed\'s static assets!\n'
            )
            sys.exit(1)

        self._move_statics()

    def _build_static(self):
        os.chdir('./seed_static')

        log.info('Seed static start install node modules.')
        self._run_command(['npm', 'install'])

        log.info('Seed static start build static files.')
        self._run_command(['npm', 'run', 'buildProd'])

        log.info('Seed static start build finish.')
        os.chdir('..')

    def _move_statics(self):
        source = os.path.join(*'./seed_static/dist'.split('/'))
        target = os.path.join(*'./src/seed/static'.split('/'))

        files = os.listdir(target)
        for file in files:
            if file in ['.gitignore']:
                continue

            if os.path.isdir(os.path.join(target, file)):
                shutil.rmtree(os.path.join(target, file))
            else:
                os.remove(os.path.join(target, file))

        files = os.listdir(source)
        for file in files:
            shutil.move(os.path.join(source, file), os.path.join(target, file))

        log.info('Move static files to seed finish.')



import os
import sys

DEFAULT_HOST = '127.0.0.1'
DEFAULT_PORT = 5000


def convert_options_to_uwsgi_env(options):
    for key, value in options.items():
        key = 'UWSGI_' + key.upper().replace('-', '_')

        if isinstance(value, int):
            value = str(value)

        yield key, value


class SeedWSGIHttpServer(object):
    def __init__(self, debug=False, workers=None):

        options = {}
        options.setdefault('module', 'seed.services.app:application')
        options.setdefault('protocol', 'http')
        options.setdefault('workers', 3)
        options.setdefault('threads', 4)
        options.setdefault('http-timeout', 60)
        options.setdefault('need-app', True)
        options.setdefault('virtualenv', sys.prefix)
        options.setdefault('die-on-term', True)
        options.setdefault(
            'log-format',
            '%(addr) - %(user) [%(ltime)] "%(method) %(uri) %(proto)" %(status) %(size) "%(referer)" "%(uagent)"'
        )

        from seed.services.app import application
        options.setdefault('%s-socket' % options['protocol'], '%s:%s' % (
            application.config.get('HOST', DEFAULT_HOST),
            application.config.get('PORT', DEFAULT_PORT)
        ))

        options['master'] = True
        options['enable-threads'] = True

        self.options = options

    def prepare_environment(self, env=None):
        if not env:
            env = os.environ

        for key, value in convert_options_to_uwsgi_env(self.options):
            env.setdefault(key, value)

        virtualenv_path = os.path.dirname(os.path.abspath(sys.argv[0]))
        current_path = str(os.path.realpath(__file__))
        if virtualenv_path not in current_path:
            env['PATH'] = '%s:%s' % (virtualenv_path, current_path)

    def run(self):
        self.prepare_environment()
        os.execvp('uwsgi', ('uwsgi', ))



from __future__ import absolute_import

import os
import sys

from distutils import log
from subprocess import check_output
from distutils.core import Command


class BaseBuildCommand(Command):
    user_options = [
        ('work-path', 'w', 'The working directory for source files. Default is .')
    ]

    def initialize_options(self):
        self.work_path = '.'

    def finalize_options(self):
        pass

    def update_mainfests(self):
        pass

    def _setup_git(self):
        work_path = self.work_path
        if os.path.exists(os.path.join(work_path, '.git')):
            log.info("initializing git submodules")
            self._run_command(['git', 'submodule', 'init'])
            self._run_command(['git', 'submodule', 'update'])

    def _setup_npm(self):
        node_version = []

        log.info("setup node and npm.....")
        for app in ['node', 'npm']:
            try:
                log.info('testing %s version.....' % app)
                node_version.append(self._run_command([app, '--version']).rstrip())
            except OSError as e:
                log.fatal(
                    'Cannot find {app} excutable. Please install {app}'
                    'and try again.'.format(app=app)
                )
                sys.exit(1)

        if node_version[0] >= b'v6.8.1':
            log.fatal('The node version need v7.8.1, the current node version is %s' % node_version[0])
            sys.exit(1)

        log.info('using node ({0}) and npm ({1})'.format(*node_version))
        self._run_command(['npm', 'install'])

    def _run_command(self, cmd, env=None):
        log.debug('running [%s]' % (' '.join(cmd), ))
        try:
            return check_output(cmd, cwd=self.work_path, env=env, shell=True)
        except Exception:
            log.error('command failed [%s] via [%s]' % (' '.join(cmd), self.work_path, ))
            raise

    def sub_commands(self):
        pass

    def run(self):
        self._setup_git()
        self._setup_npm()
        self._build()
        self.update_mainfests()



DEFUALT_RETRY_COUNT = 3


class BaseDrive(object):
    def __init__(self, ip, port, name, user, password):
        self.ip = ip
        self.port = port
        self.name = name
        self.user = user
        self.password = password

        self._connect()

    def query(self, sql, params=[], retry_count=1):
        raise NotImplementedError("Need to implemented!")

    def execute(self, sql, params=[]):
        raise NotImplementedError("Need to implemented!")

    def test_connection(self):
        rows = self.query("SELECT 1")
        return len(rows) == 1

    def _connect(self):
        raise NotImplementedError("Need to implemented!")

    def _raise_retry_count(self, retry_count):
        if retry_count < 0:
            raise Exception("Retry time out")

    def _get_connection(self):
        if self.alive():
            self._connect()

    def _commit(self):
        """ 完成SQL执行
        """
        self.conn.commit()

    def _rollback(self):
        """ 回滚SQL
        """
        self.conn.rollback()

    def _gen_cursor(self):
        cursor = self.conn.cursor()
        return cursor

    def alive(self):
        """ 测试连接是否还存在
        """
        if self.conn:
            return True if self.conn.closed == 0 else False

        return False

    def close(self):
        """ 关闭连接
        """
        self.conn.close()
